@@ -1,4 +1,6 @@
-use crate::esruntime::EsRuntime;
+//! contains the legacy EsValueFacade which will be deprecated in favour of utils::JsValueFacade in the near future
+
+use crate::facades::{QuickJsRuntimeFacade, QuickjsRuntimeFacadeInner};
 use crate::quickjs_utils::arrays::{get_element_q, get_length_q, is_array_q};
 use crate::quickjs_utils::dates::is_date_q;
 use crate::quickjs_utils::errors::{error_to_js_error, is_error_q};
@@ -8,99 +10,29 @@ use crate::quickjs_utils::objects::{get_property_names_q, get_property_q};
 use crate::quickjs_utils::primitives::to_string_q;
 use crate::quickjs_utils::promises::{is_promise_q, PromiseRef};
 use crate::quickjs_utils::{functions, new_null_ref, promises};
-use crate::quickjscontext::QuickJsContext;
-use crate::quickjsruntime::QuickJsRuntime;
+use crate::quickjsrealmadapter::QuickJsRealmAdapter;
+use crate::quickjsruntimeadapter::QuickJsRuntimeAdapter;
 use crate::reflection;
 use crate::valueref::*;
 use futures::executor::block_on;
-use futures::task::{Context, Poll};
 use hirofa_utils::auto_id_map::AutoIdMap;
 use hirofa_utils::debug_mutex::DebugMutex;
 use hirofa_utils::js_utils::JsError;
+use hirofa_utils::resolvable_future::{ResolvableFuture, ResolvableFutureResolver};
 use std::cell::RefCell;
 use std::collections::HashMap;
 use std::fmt::{Debug, Error, Formatter};
 use std::future::Future;
-use std::pin::Pin;
 use std::rc::Rc;
-use std::sync::mpsc::{sync_channel, Receiver, SendError, SyncSender};
 use std::sync::{Arc, Mutex, Weak};
-use std::task::Waker;
 
-pub struct TaskFutureResolver<R> {
-    sender: SyncSender<R>,
-    waker: DebugMutex<Option<Waker>>,
-}
-
-impl<R> TaskFutureResolver<R> {
-    pub fn new(tx: SyncSender<R>) -> Self {
-        Self {
-            sender: tx,
-            waker: DebugMutex::new(None, "TaskFutureResolver::waker"),
-        }
-    }
-    pub fn resolve(&self, resolution: R) -> Result<(), SendError<R>> {
-        log::trace!("TaskFutureResolver.resolve");
-        self.sender.send(resolution)?;
-
-        let waker_opt = &mut *self.waker.lock("resolve2").unwrap();
-        if let Some(waker) = waker_opt.take() {
-            waker.wake();
-        }
-        Ok(())
-    }
-}
-
-pub struct TaskFuture<R> {
-    result: Receiver<R>,
-    resolver: Arc<TaskFutureResolver<R>>,
-}
-impl<R> TaskFuture<R> {
-    pub fn new() -> Self {
-        let (tx, rx) = sync_channel(1);
-
-        Self {
-            result: rx,
-            resolver: Arc::new(TaskFutureResolver::new(tx)),
-        }
-    }
-    pub fn get_resolver(&self) -> Arc<TaskFutureResolver<R>> {
-        self.resolver.clone()
-    }
-}
-impl<R> Default for TaskFuture<R> {
-    fn default() -> Self {
-        Self::new()
-    }
-}
-impl<R> Future for TaskFuture<R> {
-    type Output = R;
-
-    fn poll(self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Self::Output> {
-        log::trace!("TaskFuture::poll");
-        match self.result.try_recv() {
-            Ok(res) => {
-                log::trace!("TaskFuture::poll -> Ready");
-                Poll::Ready(res)
-            }
-            Err(_) => {
-                log::trace!("TaskFuture::poll -> Pending");
-                let mtx = &self.resolver.waker;
-                let waker_opt = &mut *mtx.lock("poll").unwrap();
-                let _ = waker_opt.replace(cx.waker().clone());
-                Poll::Pending
-            }
-        }
-    }
-}
-
-pub type EsValueFacadeFuture<R, E> = TaskFuture<Result<R, E>>;
+pub type EsValueFacadeFuture<R, E> = ResolvableFuture<Result<R, E>>;
 
 pub type PromiseReactionType =
     Option<Box<dyn Fn(EsValueFacade) -> Result<EsValueFacade, JsError> + Send + 'static>>;
 
 pub trait EsValueConvertible {
-    fn as_js_value(&mut self, q_ctx: &QuickJsContext) -> Result<JSValueRef, JsError>;
+    fn as_js_value(&mut self, q_ctx: &QuickJsRealmAdapter) -> Result<JSValueRef, JsError>;
 
     #[allow(clippy::wrong_self_convention)]
     fn to_es_value_facade(self) -> EsValueFacade
@@ -229,7 +161,7 @@ impl EsProxyInstance {
 }
 
 impl EsValueConvertible for EsProxyInstance {
-    fn as_js_value(&mut self, q_ctx: &QuickJsContext) -> Result<JSValueRef, JsError> {
+    fn as_js_value(&mut self, q_ctx: &QuickJsRealmAdapter) -> Result<JSValueRef, JsError> {
         let proxy_opt = reflection::get_proxy(q_ctx, self.class_name);
         if let Some(proxy) = proxy_opt {
             reflection::new_instance3(&proxy, self.instance_id, q_ctx)
@@ -243,7 +175,7 @@ impl EsValueConvertible for EsProxyInstance {
 }
 
 impl EsValueConvertible for EsNullValue {
-    fn as_js_value(&mut self, _q_ctx: &QuickJsContext) -> Result<JSValueRef, JsError> {
+    fn as_js_value(&mut self, _q_ctx: &QuickJsRealmAdapter) -> Result<JSValueRef, JsError> {
         Ok(crate::quickjs_utils::new_null_ref())
     }
 
@@ -261,7 +193,7 @@ impl EsValueConvertible for EsNullValue {
 }
 
 impl EsValueConvertible for EsUndefinedValue {
-    fn as_js_value(&mut self, _q_ctx: &QuickJsContext) -> Result<JSValueRef, JsError> {
+    fn as_js_value(&mut self, _q_ctx: &QuickJsRealmAdapter) -> Result<JSValueRef, JsError> {
         Ok(crate::quickjs_utils::new_undefined_ref())
     }
 
@@ -293,14 +225,14 @@ enum EsType {
 struct CachedJSValueRef {
     cached_obj_id: i32,
     context_id: String,
-    es_rt: Weak<EsRuntime>,
+    rti_ref: Weak<QuickjsRuntimeFacadeInner>,
     es_type: EsType,
 }
 
 impl CachedJSValueRef {
-    fn new(q_ctx: &QuickJsContext, value_ref: &JSValueRef) -> Self {
+    fn new(q_ctx: &QuickJsRealmAdapter, value_ref: &JSValueRef) -> Self {
         log::trace!("> CachedJSValueRef::new");
-        let es_rt = QuickJsRuntime::do_with(|q_js_rt| q_js_rt.get_rt_ref().unwrap());
+        let el_ref = QuickJsRuntimeAdapter::do_with(|q_js_rt| q_js_rt.get_rti_ref().unwrap());
         let cached_obj_id = q_ctx.cache_object(value_ref.clone());
 
         let es_type = if value_ref.is_big_int() {
@@ -322,7 +254,7 @@ impl CachedJSValueRef {
         let ret = Self {
             cached_obj_id,
             context_id: q_ctx.id.to_string(),
-            es_rt: Arc::downgrade(&es_rt),
+            rti_ref: Arc::downgrade(&el_ref),
             es_type,
         };
         log::trace!("< CachedJSValueRef::new");
@@ -331,13 +263,13 @@ impl CachedJSValueRef {
 
     fn do_with_sync<C, R: Send + 'static>(&self, consumer: C) -> R
     where
-        C: FnOnce(&QuickJsRuntime, &QuickJsContext, JSValueRef) -> R + Send + 'static,
+        C: FnOnce(&QuickJsRuntimeAdapter, &QuickJsRealmAdapter, JSValueRef) -> R + Send + 'static,
     {
         let cached_obj_id = self.cached_obj_id;
 
-        if let Some(es_rt) = self.es_rt.upgrade() {
+        if let Some(el_ref) = self.rti_ref.upgrade() {
             let context_id_then = self.context_id.clone();
-            es_rt.exe_rt_task_in_event_loop(move |q_js_rt| {
+            el_ref.exe_rt_task_in_event_loop(move |q_js_rt| {
                 let q_ctx = q_js_rt.get_context(context_id_then.as_str());
 
                 q_ctx.with_cached_obj(cached_obj_id, |cached_obj_ref| {
@@ -351,18 +283,20 @@ impl CachedJSValueRef {
 
     fn do_with_async<C>(&self, consumer: C)
     where
-        C: FnOnce(&QuickJsRuntime, &QuickJsContext, JSValueRef) + Send + 'static,
+        C: FnOnce(&QuickJsRuntimeAdapter, &QuickJsRealmAdapter, JSValueRef) + Send + 'static,
     {
         let cached_obj_id = self.cached_obj_id;
 
-        if let Some(es_rt) = self.es_rt.upgrade() {
+        if let Some(rti_ref) = self.rti_ref.upgrade() {
             let context_id_then = self.context_id.clone();
-            es_rt.add_rt_task_to_event_loop_void(move |q_js_rt| {
-                let q_ctx = q_js_rt.get_context(context_id_then.as_str());
-
-                q_ctx.with_cached_obj(cached_obj_id, |cached_obj_ref| {
-                    consumer(q_js_rt, q_ctx, cached_obj_ref);
-                });
+            rti_ref.add_rt_task_to_event_loop_void(move |q_js_rt| {
+                if let Some(q_ctx) = q_js_rt.opt_context(context_id_then.as_str()) {
+                    q_ctx.with_cached_obj(cached_obj_id, |cached_obj_ref| {
+                        consumer(q_js_rt, q_ctx, cached_obj_ref);
+                    });
+                } else {
+                    log::error!("do_with_async failed, no such context: {}", context_id_then)
+                }
             });
         } else {
             panic!("rt was dropped");
@@ -372,13 +306,14 @@ impl CachedJSValueRef {
 
 impl Drop for CachedJSValueRef {
     fn drop(&mut self) {
-        if let Some(rt_arc) = self.es_rt.upgrade() {
+        if let Some(rti_ref) = self.rti_ref.upgrade() {
             let cached_obj_id = self.cached_obj_id;
             let context_id = self.context_id.clone();
-            rt_arc.add_rt_task_to_event_loop_void(move |q_js_rt| {
+            rti_ref.add_rt_task_to_event_loop_void(move |q_js_rt| {
                 if q_js_rt.has_context(context_id.as_str()) {
-                    let q_ctx = q_js_rt.get_context(context_id.as_str());
-                    q_ctx.remove_cached_obj_if_present(cached_obj_id);
+                    if let Some(q_ctx) = q_js_rt.opt_context(context_id.as_str()) {
+                        q_ctx.remove_cached_obj_if_present(cached_obj_id);
+                    }
                 }
             });
         }
@@ -386,15 +321,17 @@ impl Drop for CachedJSValueRef {
 }
 
 fn pipe_promise_resolution_to_sender(
-    q_ctx: &QuickJsContext,
+    q_ctx: &QuickJsRealmAdapter,
     prom_obj_ref: &JSValueRef,
-    tx: Arc<TaskFutureResolver<Result<EsValueFacade, EsValueFacade>>>,
+    tx: Arc<ResolvableFutureResolver<Result<EsValueFacade, EsValueFacade>>>,
 ) {
+    log::trace!("pipe_promise_resolution_to_sender");
     let tx2 = tx.clone();
     let then_func_ref = functions::new_function_q(
         q_ctx,
         "promise_then_result_transmitter",
         move |q_ctx, _this_ref, args| {
+            log::trace!("pipe_promise_resolution_to_sender.then_func");
             // these clones are needed because create_func requires a Fn and not a FnOnce
             // in practice however the Fn is called only once
             let tx3 = tx2.clone();
@@ -431,6 +368,7 @@ fn pipe_promise_resolution_to_sender(
         q_ctx,
         "promise_catch_result_transmitter",
         move |q_ctx, _this_ref, args| {
+            log::trace!("pipe_promise_resolution_to_sender.catch_func");
             // these clones are needed because create_func requires a Fn and not a FnOnce
             // in practice however the Fn is called only once
             let tx3 = tx.clone();
@@ -473,7 +411,7 @@ fn pipe_promise_resolution_to_sender(
 }
 
 impl EsValueConvertible for CachedJSValueRef {
-    fn as_js_value(&mut self, q_ctx: &QuickJsContext) -> Result<JSValueRef, JsError> {
+    fn as_js_value(&mut self, q_ctx: &QuickJsRealmAdapter) -> Result<JSValueRef, JsError> {
         Ok(q_ctx.with_cached_obj(self.cached_obj_id, |obj_ref| obj_ref))
     }
 
@@ -594,7 +532,7 @@ impl EsValueConvertible for CachedJSValueRef {
 
     fn get_promise_result(&self) -> EsValueFacadeFuture<EsValueFacade, EsValueFacade> {
         assert!(self.is_promise());
-        let fut = TaskFuture::new();
+        let fut = EsValueFacadeFuture::new();
         let tx = fut.get_resolver();
         self.do_with_async(move |_q_js_rt, q_ctx, prom_obj_ref| {
             pipe_promise_resolution_to_sender(q_ctx, &prom_obj_ref, tx);
@@ -613,7 +551,7 @@ impl EsValueConvertible for CachedJSValueRef {
             let then_ref = if let Some(then_fn) = then {
                 let then_fn_rc = Rc::new(then_fn);
 
-                let then_fn_raw = move |q_ctx: &QuickJsContext,
+                let then_fn_raw = move |q_ctx: &QuickJsRealmAdapter,
                                         _this_ref: &JSValueRef,
                                         args_ref: &[JSValueRef]| {
                     let then_fn_rc = then_fn_rc.clone();
@@ -638,7 +576,7 @@ impl EsValueConvertible for CachedJSValueRef {
                 let t = functions::new_function_q(
                     q_ctx,
                     "",
-                    move |q_ctx: &QuickJsContext, _this_ref, args_ref: &[JSValueRef]| {
+                    move |q_ctx: &QuickJsRealmAdapter, _this_ref, args_ref: &[JSValueRef]| {
                         let val_ref = &args_ref[0];
                         let catch_fn_rc = catch_fn_rc.clone();
 
@@ -660,7 +598,7 @@ impl EsValueConvertible for CachedJSValueRef {
                 let t = functions::new_function_q(
                     q_ctx,
                     "",
-                    move |_q_ctx: &QuickJsContext, _this_ref, _args_ref| {
+                    move |_q_ctx: &QuickJsRealmAdapter, _this_ref, _args_ref| {
                         finally_fn();
                         Ok(crate::quickjs_utils::new_null_ref())
                     },
@@ -746,7 +684,7 @@ impl EsValueConvertible for CachedJSValueRef {
 }
 
 impl EsValueConvertible for String {
-    fn as_js_value(&mut self, q_ctx: &QuickJsContext) -> Result<JSValueRef, JsError> {
+    fn as_js_value(&mut self, q_ctx: &QuickJsRealmAdapter) -> Result<JSValueRef, JsError> {
         crate::quickjs_utils::primitives::from_string_q(q_ctx, self.as_str())
     }
 
@@ -768,7 +706,7 @@ impl EsValueConvertible for String {
 }
 
 impl EsValueConvertible for i32 {
-    fn as_js_value(&mut self, _q_ctx: &QuickJsContext) -> Result<JSValueRef, JsError> {
+    fn as_js_value(&mut self, _q_ctx: &QuickJsRealmAdapter) -> Result<JSValueRef, JsError> {
         Ok(crate::quickjs_utils::primitives::from_i32(*self))
     }
 
@@ -790,7 +728,7 @@ impl EsValueConvertible for i32 {
 }
 
 impl EsValueConvertible for bool {
-    fn as_js_value(&mut self, _q_ctx: &QuickJsContext) -> Result<JSValueRef, JsError> {
+    fn as_js_value(&mut self, _q_ctx: &QuickJsRealmAdapter) -> Result<JSValueRef, JsError> {
         Ok(crate::quickjs_utils::primitives::from_bool(*self))
     }
 
@@ -812,7 +750,7 @@ impl EsValueConvertible for bool {
 }
 
 impl EsValueConvertible for f64 {
-    fn as_js_value(&mut self, _q_ctx: &QuickJsContext) -> Result<JSValueRef, JsError> {
+    fn as_js_value(&mut self, _q_ctx: &QuickJsRealmAdapter) -> Result<JSValueRef, JsError> {
         Ok(crate::quickjs_utils::primitives::from_f64(*self))
     }
     fn is_f64(&self) -> bool {
@@ -833,7 +771,7 @@ impl EsValueConvertible for f64 {
 }
 
 impl EsValueConvertible for Vec<EsValueFacade> {
-    fn as_js_value(&mut self, q_ctx: &QuickJsContext) -> Result<JSValueRef, JsError> {
+    fn as_js_value(&mut self, q_ctx: &QuickJsRealmAdapter) -> Result<JSValueRef, JsError> {
         // create the array
 
         let arr = crate::quickjs_utils::arrays::create_array_q(q_ctx)
@@ -846,14 +784,14 @@ impl EsValueConvertible for Vec<EsValueFacade> {
 
             let item_val_ref = item.as_js_value(q_ctx)?;
 
-            crate::quickjs_utils::arrays::set_element_q(q_ctx, &arr, index as u32, item_val_ref)?;
+            crate::quickjs_utils::arrays::set_element_q(q_ctx, &arr, index as u32, &item_val_ref)?;
         }
         Ok(arr)
     }
 }
 
 impl EsValueConvertible for HashMap<String, EsValueFacade> {
-    fn as_js_value(&mut self, q_ctx: &QuickJsContext) -> Result<JSValueRef, JsError> {
+    fn as_js_value(&mut self, q_ctx: &QuickJsRealmAdapter) -> Result<JSValueRef, JsError> {
         // create new obj
         let obj_ref = crate::quickjs_utils::objects::create_object_q(q_ctx)
             .ok()
@@ -886,7 +824,7 @@ thread_local! {
 }
 
 struct EsPromiseResolvableHandleInfo {
-    weak_es_rt: Weak<EsRuntime>,
+    weak_rti_ref: Weak<QuickjsRuntimeFacadeInner>,
     id: usize,
     context_id: String,
 }
@@ -929,7 +867,7 @@ impl EsPromiseResolvableHandle {
                 // resolve
                 let id = info.id;
                 let context_id = info.context_id.clone();
-                if let Some(rt) = info.weak_es_rt.upgrade() {
+                if let Some(rt) = info.weak_rti_ref.upgrade() {
                     Some((rt, id, context_id, value))
                 } else {
                     log::error!("rt was dropped while resolving");
@@ -941,23 +879,26 @@ impl EsPromiseResolvableHandle {
                 None
             }
         });
-        if let Some((es_rt, id, context_id, mut value)) = rt_opt {
-            es_rt.add_rt_task_to_event_loop_void(move |q_js_rt| {
+        if let Some((rti_ref, id, context_id, mut value)) = rt_opt {
+            rti_ref.add_rt_task_to_event_loop_void(move |q_js_rt| {
                 log::trace!("resolving handle with val, stage 2: {:?}", value);
-                let q_ctx = q_js_rt.get_context(context_id.as_str());
-                ESPROMISE_REFS.with(move |rc| {
-                    let map = &*rc.borrow();
-                    let p_ref = map.get(&id).expect("no such promise");
+                if let Some(q_ctx) = q_js_rt.opt_context(context_id.as_str()) {
+                    ESPROMISE_REFS.with(move |rc| {
+                        let map = &*rc.borrow();
+                        let p_ref = map.get(&id).expect("no such promise");
 
-                    let js_val = value
-                        .as_js_value(q_ctx)
-                        .ok()
-                        .expect("could not convert to JSValue");
-                    let resolve_res = unsafe { p_ref.resolve(q_ctx.context, js_val) };
-                    if resolve_res.is_err() {
-                        log::error!("resolve failed: {}", resolve_res.err().unwrap());
-                    }
-                });
+                        let js_val = value
+                            .as_js_value(q_ctx)
+                            .ok()
+                            .expect("could not convert to JSValue");
+                        let resolve_res = unsafe { p_ref.resolve(q_ctx.context, js_val) };
+                        if resolve_res.is_err() {
+                            log::error!("resolve failed: {}", resolve_res.err().unwrap());
+                        }
+                    });
+                } else {
+                    log::error!("resolve failed: no such context {}", context_id);
+                }
             });
         }
     }
@@ -969,7 +910,7 @@ impl EsPromiseResolvableHandle {
                 let id = info.id;
                 let context_id = info.context_id.clone();
 
-                if let Some(rt) = info.weak_es_rt.upgrade() {
+                if let Some(rt) = info.weak_rti_ref.upgrade() {
                     Some((rt, id, context_id, value))
                 } else {
                     log::error!("rt was dropped while rejecting");
@@ -981,26 +922,34 @@ impl EsPromiseResolvableHandle {
                 None
             }
         });
-        if let Some((es_rt, id, context_id, mut value)) = rt_opt {
-            es_rt.add_rt_task_to_event_loop_void(move |q_js_rt| {
+        if let Some((rti_ref, id, context_id, mut value)) = rt_opt {
+            rti_ref.add_rt_task_to_event_loop_void(move |q_js_rt| {
                 log::trace!("rejecting handle with val, stage 2: {:?}", value);
-                let q_ctx = q_js_rt.get_context(context_id.as_str());
-                ESPROMISE_REFS.with(move |rc| {
-                    let map = &*rc.borrow();
-                    let p_ref = map.get(&id).expect("no such promise");
-                    let js_val = value
-                        .as_js_value(q_ctx)
-                        .ok()
-                        .expect("could not convert to JSValue");
-                    let reject_res = unsafe { p_ref.reject(q_ctx.context, js_val) };
-                    if reject_res.is_err() {
-                        log::error!("reject failed: {}", reject_res.err().unwrap());
-                    }
-                });
+                if let Some(q_ctx) = q_js_rt.opt_context(context_id.as_str()) {
+                    ESPROMISE_REFS.with(move |rc| {
+                        let map = &*rc.borrow();
+                        let p_ref = map.get(&id).expect("no such promise");
+                        let js_val = value
+                            .as_js_value(q_ctx)
+                            .ok()
+                            .expect("could not convert to JSValue");
+                        let reject_res = unsafe { p_ref.reject(q_ctx.context, js_val) };
+                        if reject_res.is_err() {
+                            log::error!("reject failed: {}", reject_res.err().unwrap());
+                        }
+                    });
+                } else {
+                    log::error!("reject failed: no such context {}", context_id);
+                }
             });
         }
     }
-    fn set_info(&self, es_rt: &Arc<EsRuntime>, id: usize, context_id: &str) -> Result<(), JsError> {
+    fn set_info(
+        &self,
+        rti_ref: Weak<QuickjsRuntimeFacadeInner>,
+        id: usize,
+        context_id: &str,
+    ) -> Result<(), JsError> {
         let resolution_opt: Option<Result<EsValueFacade, EsValueFacade>> =
             self.with_inner(|inner| {
                 if inner.js_info.is_some() {
@@ -1008,7 +957,7 @@ impl EsPromiseResolvableHandle {
                 } else {
                     // set info
                     inner.js_info = Some(EsPromiseResolvableHandleInfo {
-                        weak_es_rt: Arc::downgrade(es_rt),
+                        weak_rti_ref: rti_ref,
                         id,
                         context_id: context_id.to_string(),
                     });
@@ -1039,8 +988,8 @@ impl Drop for EsPromiseResolvableHandleInner {
     fn drop(&mut self) {
         if let Some(info) = &self.js_info {
             let id = info.id;
-            if let Some(es_rt) = info.weak_es_rt.upgrade() {
-                es_rt.add_rt_task_to_event_loop_void(move |_q_js_rt| {
+            if let Some(rti_ref) = info.weak_rti_ref.upgrade() {
+                rti_ref.add_task_to_event_loop_void(move || {
                     ESPROMISE_REFS.with(move |rc| {
                         let map = &mut *rc.borrow_mut();
                         map.remove(&id);
@@ -1101,13 +1050,13 @@ impl EsFunction {
     /// # Example
     /// ```rust
     /// use quickjs_runtime::esvalue::{EsFunction, EsValueConvertible, ES_NULL, EsValueFacade};
-    /// use quickjs_runtime::esruntimebuilder::EsRuntimeBuilder;
+    /// use quickjs_runtime::builder::QuickJsRuntimeBuilder;
     /// use hirofa_utils::js_utils::Script;
     /// async fn do_something(args: Vec<EsValueFacade>) -> Result<EsValueFacade, String> {
     ///     Ok(123.to_es_value_facade())
     /// }
     /// let func_esvf = EsFunction::new_async("my_callback", do_something).to_es_value_facade();
-    /// let rt = EsRuntimeBuilder::new().build();
+    /// let rt = QuickJsRuntimeBuilder::new().build();
     /// rt.eval_sync(Script::new("new_async.es", "this.test_func = function(cb){return cb();};")).ok().expect("func invo failed");
     /// let func_res = rt.call_function_sync(vec![], "test_func", vec![func_esvf]).ok().expect("func invo failed2");
     /// let ret = func_res.get_promise_result_sync().ok().expect("do_something returned err");
@@ -1132,7 +1081,7 @@ impl EsFunction {
 }
 
 impl EsValueConvertible for EsFunction {
-    fn as_js_value(&mut self, q_ctx: &QuickJsContext) -> Result<JSValueRef, JsError> {
+    fn as_js_value(&mut self, q_ctx: &QuickJsRealmAdapter) -> Result<JSValueRef, JsError> {
         let func_arc_mtx = self.method.clone();
 
         new_function_q(
@@ -1142,7 +1091,7 @@ impl EsValueConvertible for EsFunction {
                 let mut args_facades = vec![];
 
                 for arg_ref in args {
-                    args_facades.push(EsValueFacade::from_jsval(q_ctx, &arg_ref)?);
+                    args_facades.push(EsValueFacade::from_jsval(q_ctx, arg_ref)?);
                 }
 
                 let func = &*func_arc_mtx.lock().unwrap();
@@ -1158,13 +1107,13 @@ impl EsValueConvertible for EsFunction {
 /// can be used to create a new Promise which is resolved with the resolver function
 /// # Example
 /// ```rust
-/// use quickjs_runtime::esruntimebuilder::EsRuntimeBuilder;
+/// use quickjs_runtime::builder::QuickJsRuntimeBuilder;
 /// use quickjs_runtime::esvalue::{EsPromise, EsValueConvertible};
 /// use std::time::Duration;
 /// use hirofa_utils::js_utils::Script;
 /// use log::LevelFilter;
 ///
-/// let rt = EsRuntimeBuilder::new().build();
+/// let rt = QuickJsRuntimeBuilder::new().build();
 /// rt.set_function(vec!["my", "comp"], "create_prom", |_q_ctx, _args| {
 ///     Ok(EsPromise::new(|| {
 ///         std::thread::sleep(Duration::from_secs(1));
@@ -1187,7 +1136,7 @@ impl EsPromise {
         let ret = Self::new_unresolving();
 
         let handle = ret.get_handle();
-        EsRuntime::add_helper_task(move || {
+        QuickJsRuntimeFacade::add_helper_task(move || {
             let val = resolver();
             match val {
                 Ok(v) => {
@@ -1206,7 +1155,7 @@ impl EsPromise {
     /// this can be used to implement a resolver which in turn used .await to get results of other async functions
     /// # Example
     /// ```rust
-    /// use quickjs_runtime::esruntimebuilder::EsRuntimeBuilder;
+    /// use quickjs_runtime::builder::QuickJsRuntimeBuilder;
     /// use quickjs_runtime::esvalue::{EsPromise, EsValueConvertible};
     /// use hirofa_utils::js_utils::Script;
     /// use std::time::Duration;
@@ -1216,7 +1165,7 @@ impl EsPromise {
     ///     i * 3
     /// }
     ///
-    /// let rt = EsRuntimeBuilder::new().build();
+    /// let rt = QuickJsRuntimeBuilder::new().build();
     ///
     /// rt.set_function(vec!["com", "my"], "testasyncfunc", |_q_ctx, args| {
     ///     let input = args[0].get_i32();
@@ -1244,7 +1193,7 @@ impl EsPromise {
 
         let handle = ret.get_handle();
 
-        let _ = EsRuntime::add_helper_task_async(async move {
+        let _ = QuickJsRuntimeFacade::add_helper_task_async(async move {
             let val = resolver.await;
             match val {
                 Ok(v) => {
@@ -1262,11 +1211,11 @@ impl EsPromise {
     /// this achieved by creating a Handle which is wrapped in an Arc and thus may be passed to another thread
     /// # Example
     /// ```rust
-    /// use quickjs_runtime::esruntimebuilder::EsRuntimeBuilder;
+    /// use quickjs_runtime::builder::QuickJsRuntimeBuilder;
     /// use hirofa_utils::js_utils::Script;
     /// use std::time::Duration;
     /// use quickjs_runtime::esvalue::{EsPromise, EsValueConvertible};
-    /// let rt = EsRuntimeBuilder::new().build();
+    /// let rt = QuickJsRuntimeBuilder::new().build();
     /// // prep a function which reacts to a promise
     /// rt.eval_sync(Script::new("new_unresolving.es", "this.new_unresolving = function(prom){prom.then((res) => {console.log('promise resolved to %s', res);});};")).ok().expect("script failed");
     /// // prep a EsPromise object
@@ -1296,7 +1245,7 @@ impl EsPromise {
 }
 
 impl EsValueConvertible for EsPromise {
-    fn as_js_value(&mut self, q_ctx: &QuickJsContext) -> Result<JSValueRef, JsError> {
+    fn as_js_value(&mut self, q_ctx: &QuickJsRealmAdapter) -> Result<JSValueRef, JsError> {
         log::trace!("EsPromise::to_js_value");
 
         let prom_ref = promises::new_promise_q(q_ctx)?;
@@ -1306,9 +1255,10 @@ impl EsValueConvertible for EsPromise {
             let map = &mut *rc.borrow_mut();
             map.insert(prom_ref)
         });
-        let es_rt = QuickJsRuntime::do_with(|q_js_rt| q_js_rt.get_rt_ref().unwrap());
+        let el_ref = QuickJsRuntimeAdapter::do_with(|q_js_rt| q_js_rt.get_rti_ref().unwrap());
 
-        self.handle.set_info(&es_rt, id, q_ctx.id.as_str())?;
+        self.handle
+            .set_info(Arc::downgrade(&el_ref), id, q_ctx.id.as_str())?;
 
         Ok(ret)
     }
@@ -1325,12 +1275,15 @@ impl EsValueFacade {
     }
 
     /// convert the value to a JSValueRef
-    pub fn as_js_value(&mut self, q_ctx: &QuickJsContext) -> Result<JSValueRef, JsError> {
+    pub fn as_js_value(&mut self, q_ctx: &QuickJsRealmAdapter) -> Result<JSValueRef, JsError> {
         self.convertible.as_js_value(q_ctx)
     }
 
     /// convert a JSValueRef to an EsValueFacade
-    pub fn from_jsval(q_ctx: &QuickJsContext, value_ref: &JSValueRef) -> Result<Self, JsError> {
+    pub fn from_jsval(
+        q_ctx: &QuickJsRealmAdapter,
+        value_ref: &JSValueRef,
+    ) -> Result<Self, JsError> {
         log::trace!("EsValueFacade::from_jsval: tag:{}", value_ref.get_tag());
 
         let r = value_ref.borrow_value();
@@ -1499,7 +1452,7 @@ impl EsValueFacade {
     /// The Result will be an Ok if the Promise was resolved or an Err if the Promise was rejected
     /// # Example
     /// ```rust
-    /// use quickjs_runtime::esruntimebuilder::EsRuntimeBuilder;
+    /// use quickjs_runtime::builder::QuickJsRuntimeBuilder;
     /// use hirofa_utils::js_utils::Script;
     /// use futures::executor::block_on;
     /// use quickjs_runtime::esvalue::EsValueFacade;
@@ -1509,7 +1462,7 @@ impl EsValueFacade {
     ///    return res_esvf.get_i32();
     /// }
     ///
-    /// let rt = EsRuntimeBuilder::new().build();
+    /// let rt = QuickJsRuntimeBuilder::new().build();
     /// let esvf = rt.eval_sync(Script::new("test_async_prom,es", "(new Promise((resolve, reject) => {setTimeout(() => {resolve(1360)}, 1000);}));")).ok().expect("script failed");
     /// let i = block_on(test_async(esvf));
     /// assert_eq!(i, 1360);
@@ -1558,10 +1511,10 @@ impl Debug for EsValueFacade {
 
 #[cfg(test)]
 pub mod tests {
-    use crate::esruntime::tests::init_test_rt;
-    use crate::esruntime::EsRuntime;
-    use crate::esruntimebuilder::EsRuntimeBuilder;
+    use crate::builder::QuickJsRuntimeBuilder;
     use crate::esvalue::{EsPromise, EsValueConvertible, EsValueFacade};
+    use crate::facades::tests::init_test_rt;
+    use crate::facades::QuickJsRuntimeFacade;
     use futures::executor::block_on;
     use hirofa_utils::js_utils::Script;
     use std::sync::{Arc, Weak};
@@ -1575,7 +1528,7 @@ pub mod tests {
 
     #[test]
     fn test_async_func() {
-        let rt: Arc<EsRuntime> = init_test_rt();
+        let rt = init_test_rt();
         let func_esvf = rt
             .eval_sync(Script::new(
                 "test_async_func.es",
@@ -1589,7 +1542,7 @@ pub mod tests {
 
     #[test]
     fn test_promise() {
-        let rt: Arc<EsRuntime> = init_test_rt();
+        let rt = init_test_rt();
         let res = rt.eval_sync(Script::new(
             "test_promise.es",
             "(new Promise(function(resolve, reject){resolve(537);}));",
@@ -1622,7 +1575,7 @@ pub mod tests {
             return res_esvf.get_i32();
         }
 
-        let rt = EsRuntimeBuilder::new().build();
+        let rt = QuickJsRuntimeBuilder::new().build();
         let esvf = rt
             .eval_sync(Script::new(
                 "test_async_prom,es",
@@ -1634,7 +1587,7 @@ pub mod tests {
         assert_eq!(i, 1360);
     }
 
-    async fn a(i: i32, rt_ref: Weak<EsRuntime>) -> i32 {
+    async fn a(i: i32, rt_ref: Weak<QuickJsRuntimeFacade>) -> i32 {
         let rt = rt_ref.upgrade().unwrap();
         let second_prom = rt
             .eval(Script::new(
@@ -1656,7 +1609,7 @@ pub mod tests {
 
     #[test]
     fn test_promise_async2() {
-        let rt = EsRuntimeBuilder::new().build();
+        let rt = Arc::new(QuickJsRuntimeBuilder::new().build());
         let rt_ref = Arc::downgrade(&rt);
         rt.set_function(vec!["com", "my"], "testasyncfunc", move |_q_ctx, args| {
             let rt_ref = rt_ref.clone();
