@@ -1,6 +1,6 @@
 use crate::quickjs_utils;
 use crate::quickjs_utils::{functions, get_global, objects, parse_args, primitives};
-use crate::quickjsruntime::QuickJsRuntime;
+use crate::quickjsruntimeadapter::QuickJsRuntimeAdapter;
 use hirofa_utils::eventloop::EventLoop;
 use hirofa_utils::js_utils::JsError;
 use libquickjs_sys as q;
@@ -9,43 +9,58 @@ use std::time::Duration;
 /// provides the setImmediate methods for the runtime
 /// # Example
 /// ```rust
-/// use quickjs_runtime::esruntimebuilder::EsRuntimeBuilder;
+/// use quickjs_runtime::builder::QuickJsRuntimeBuilder;
 /// use hirofa_utils::js_utils::Script;
 /// use std::time::Duration;
-/// let rt = EsRuntimeBuilder::new().build();
+/// let rt = QuickJsRuntimeBuilder::new().build();
 /// rt.eval(Script::new("test_timeout.es", "setTimeout(() => {console.log('timed logging')}, 1000);"));
 /// std::thread::sleep(Duration::from_secs(2));
 /// ```
-
-pub fn init(q_js_rt: &QuickJsRuntime) -> Result<(), JsError> {
+pub fn init(q_js_rt: &QuickJsRuntimeAdapter) -> Result<(), JsError> {
     log::trace!("set_timeout::init");
 
     q_js_rt.add_context_init_hook(|_q_js_rt, q_ctx| {
-        let set_timeout_func =
-            functions::new_native_function_q(q_ctx, "setTimeout", Some(set_timeout), 2, false)?;
-        let set_interval_func =
-            functions::new_native_function_q(q_ctx, "setInterval", Some(set_interval), 2, false)?;
-        let clear_timeout_func =
-            functions::new_native_function_q(q_ctx, "clearTimeout", Some(clear_timeout), 1, false)?;
-        let clear_interval_func = functions::new_native_function_q(
-            q_ctx,
-            "clearInterval",
-            Some(clear_interval),
-            1,
-            false,
-        )?;
-
         let global = unsafe { get_global(q_ctx.context) };
+        #[cfg(feature = "settimeout")]
+        {
+            let set_timeout_func =
+                functions::new_native_function_q(q_ctx, "setTimeout", Some(set_timeout), 2, false)?;
+            let clear_timeout_func = functions::new_native_function_q(
+                q_ctx,
+                "clearTimeout",
+                Some(clear_timeout),
+                1,
+                false,
+            )?;
+            objects::set_property2_q(q_ctx, &global, "setTimeout", &set_timeout_func, 0)?;
+            objects::set_property2_q(q_ctx, &global, "clearTimeout", &clear_timeout_func, 0)?;
+        }
+        #[cfg(feature = "setinterval")]
+        {
+            let set_interval_func = functions::new_native_function_q(
+                q_ctx,
+                "setInterval",
+                Some(set_interval),
+                2,
+                false,
+            )?;
+            let clear_interval_func = functions::new_native_function_q(
+                q_ctx,
+                "clearInterval",
+                Some(clear_interval),
+                1,
+                false,
+            )?;
 
-        objects::set_property2_q(q_ctx, &global, "setTimeout", &set_timeout_func, 0)?;
-        objects::set_property2_q(q_ctx, &global, "setInterval", &set_interval_func, 0)?;
-        objects::set_property2_q(q_ctx, &global, "clearTimeout", &clear_timeout_func, 0)?;
-        objects::set_property2_q(q_ctx, &global, "clearInterval", &clear_interval_func, 0)?;
+            objects::set_property2_q(q_ctx, &global, "setInterval", &set_interval_func, 0)?;
+            objects::set_property2_q(q_ctx, &global, "clearInterval", &clear_interval_func, 0)?;
+        }
         Ok(())
     })?;
     Ok(())
 }
 
+#[cfg(feature = "settimeout")]
 unsafe extern "C" fn set_timeout(
     context: *mut q::JSContext,
     _this_val: q::JSValue,
@@ -56,7 +71,7 @@ unsafe extern "C" fn set_timeout(
 
     let mut args = parse_args(context, argc, argv);
 
-    QuickJsRuntime::do_with(move |q_js_rt| {
+    QuickJsRuntimeAdapter::do_with(move |q_js_rt| {
         let q_ctx = q_js_rt.get_quickjs_context(context);
         if args.is_empty() {
             return q_ctx.report_ex("setTimeout requires at least one argument");
@@ -84,16 +99,19 @@ unsafe extern "C" fn set_timeout(
 
         let id = EventLoop::add_timeout(
             move || {
-                QuickJsRuntime::do_with(|q_js_rt| {
+                QuickJsRuntimeAdapter::do_with(|q_js_rt| {
                     let mut args = args.clone();
                     let func = args.remove(0);
-                    let q_ctx = q_js_rt.get_context(q_ctx_id.as_str());
-                    match functions::call_function_q(q_ctx, &func, args, None) {
-                        Ok(_) => {}
-                        Err(e) => {
-                            log::error!("setTimeout func failed: {}", e);
-                        }
-                    };
+                    if let Some(q_ctx) = q_js_rt.opt_context(q_ctx_id.as_str()) {
+                        match functions::call_function_q(q_ctx, &func, args, None) {
+                            Ok(_) => {}
+                            Err(e) => {
+                                log::error!("setTimeout func failed: {}", e);
+                            }
+                        };
+                    } else {
+                        log::error!("setTimeout func failed: no such context: {}", q_ctx_id);
+                    }
                     q_js_rt.run_pending_jobs_if_any();
                 })
             },
@@ -104,6 +122,7 @@ unsafe extern "C" fn set_timeout(
     })
 }
 
+#[cfg(feature = "setinterval")]
 unsafe extern "C" fn set_interval(
     context: *mut q::JSContext,
     _this_val: q::JSValue,
@@ -114,7 +133,7 @@ unsafe extern "C" fn set_interval(
 
     let mut args = parse_args(context, argc, argv);
 
-    QuickJsRuntime::do_with(|q_js_rt| {
+    QuickJsRuntimeAdapter::do_with(|q_js_rt| {
         let q_ctx = q_js_rt.get_quickjs_context(context);
         if args.is_empty() {
             return q_ctx.report_ex("setInterval requires at least one argument");
@@ -142,19 +161,21 @@ unsafe extern "C" fn set_interval(
 
         let id = EventLoop::add_interval(
             move || {
-                QuickJsRuntime::do_with(|q_js_rt| {
-                    let q_ctx = q_js_rt.get_context(q_ctx_id.as_str());
-                    let mut args = args.clone();
+                QuickJsRuntimeAdapter::do_with(|q_js_rt| {
+                    if let Some(q_ctx) = q_js_rt.opt_context(q_ctx_id.as_str()) {
+                        let mut args = args.clone();
 
-                    let func = args.remove(0);
-
-                    match functions::call_function_q(q_ctx, &func, args, None) {
-                        Ok(_) => {}
-                        Err(e) => {
-                            log::error!("setInterval func failed: {}", e);
-                        }
-                    };
+                        let func = args.remove(0);
 
+                        match functions::call_function_q(q_ctx, &func, args, None) {
+                            Ok(_) => {}
+                            Err(e) => {
+                                log::error!("setInterval func failed: {}", e);
+                            }
+                        };
+                    } else {
+                        log::error!("setInterval func failed: no such context: {}", q_ctx_id);
+                    }
                     q_js_rt.run_pending_jobs_if_any();
                 })
             },
@@ -166,6 +187,7 @@ unsafe extern "C" fn set_interval(
     })
 }
 
+#[cfg(feature = "setinterval")]
 unsafe extern "C" fn clear_interval(
     context: *mut q::JSContext,
     _this_val: q::JSValue,
@@ -175,7 +197,7 @@ unsafe extern "C" fn clear_interval(
     log::trace!("> clear_interval");
 
     let args = parse_args(context, argc, argv);
-    QuickJsRuntime::do_with(|q_js_rt| {
+    QuickJsRuntimeAdapter::do_with(|q_js_rt| {
         let q_ctx = q_js_rt.get_quickjs_context(context);
         if args.is_empty() {
             return q_ctx.report_ex("clearInterval requires at least one argument");
@@ -190,6 +212,7 @@ unsafe extern "C" fn clear_interval(
     })
 }
 
+#[cfg(feature = "settimeout")]
 unsafe extern "C" fn clear_timeout(
     context: *mut q::JSContext,
     _this_val: q::JSValue,
@@ -200,7 +223,7 @@ unsafe extern "C" fn clear_timeout(
 
     let args = parse_args(context, argc, argv);
 
-    QuickJsRuntime::do_with(move |q_js_rt| {
+    QuickJsRuntimeAdapter::do_with(move |q_js_rt| {
         let q_ctx = q_js_rt.get_quickjs_context(context);
         if args.is_empty() {
             return q_ctx.report_ex("clearTimeout requires at least one argument");
@@ -219,18 +242,16 @@ unsafe extern "C" fn clear_timeout(
 
 #[cfg(test)]
 pub mod tests {
-    use crate::esruntime::tests::init_test_rt;
-    use crate::esruntime::EsRuntime;
+    use crate::facades::tests::init_test_rt;
     use crate::quickjs_utils::get_global_q;
     use crate::quickjs_utils::objects::get_property_q;
     use crate::quickjs_utils::primitives::to_i32;
     use hirofa_utils::js_utils::Script;
-    use std::sync::Arc;
     use std::time::Duration;
 
     #[test]
     fn test_set_timeout_prom_res() {
-        let rt: Arc<EsRuntime> = init_test_rt();
+        let rt = init_test_rt();
         let esvf = rt
             .eval_sync(Script::new(
                 "test_set_timeout_prom_res.es",
@@ -257,7 +278,7 @@ pub mod tests {
 
     #[test]
     fn test_set_timeout_prom_res_nested() {
-        let rt: Arc<EsRuntime> = init_test_rt();
+        let rt = init_test_rt();
         let esvf = rt
             .eval_sync(Script::new(
                 "test_set_timeout_prom_res_nested.es",
@@ -284,7 +305,7 @@ pub mod tests {
 
     #[test]
     fn test_set_timeout() {
-        let rt: Arc<EsRuntime> = init_test_rt();
+        let rt = init_test_rt();
 
         rt.eval_sync(Script::new("test_set_interval.es", "let t_id1 = setInterval((a, b) => {console.log('setInterval invoked with %s and %s', a, b);}, 500, 123, 456);")).ok().expect("fail a");
         rt.eval_sync(Script::new("test_set_timeout.es", "let t_id2 = setTimeout((a, b) => {console.log('setTimeout1 invoked with %s and %s', a, b);}, 500, 123, 456);")).ok().expect("fail b");
