@@ -1,4 +1,4 @@
-use crate::quickjscontext::QuickJsContext;
+use crate::quickjsrealmadapter::QuickJsRealmAdapter;
 use crate::valueref::JSValueRef;
 use hirofa_utils::js_utils::JsError;
 use libquickjs_sys as q;
@@ -6,11 +6,11 @@ use libquickjs_sys as q;
 /// Check whether an object is an array
 /// # Example
 /// ```rust
-/// use quickjs_runtime::esruntimebuilder::EsRuntimeBuilder;
+/// use quickjs_runtime::builder::QuickJsRuntimeBuilder;
 /// use hirofa_utils::js_utils::Script;
 /// use quickjs_runtime::quickjs_utils::arrays;
 ///
-/// let rt = EsRuntimeBuilder::new().build();
+/// let rt = QuickJsRuntimeBuilder::new().build();
 /// rt.exe_rt_task_in_event_loop(|q_js_rt| {
 ///     let q_ctx = q_js_rt.get_main_context();
 ///     let obj_ref = q_ctx.eval(Script::new("is_array_test.es", "([1, 2, 3]);")).ok().expect("script failed");
@@ -18,7 +18,7 @@ use libquickjs_sys as q;
 ///     assert!(is_array);
 /// });
 /// ```
-pub fn is_array_q(q_ctx: &QuickJsContext, obj_ref: &JSValueRef) -> bool {
+pub fn is_array_q(q_ctx: &QuickJsRealmAdapter, obj_ref: &JSValueRef) -> bool {
     unsafe { is_array(q_ctx.context, obj_ref) }
 }
 
@@ -33,11 +33,11 @@ pub unsafe fn is_array(context: *mut q::JSContext, obj_ref: &JSValueRef) -> bool
 /// Get the length of an Array
 /// # Example
 /// ```rust
-/// use quickjs_runtime::esruntimebuilder::EsRuntimeBuilder;
+/// use quickjs_runtime::builder::QuickJsRuntimeBuilder;
 /// use hirofa_utils::js_utils::Script;
 /// use quickjs_runtime::quickjs_utils::arrays;
 ///
-/// let rt = EsRuntimeBuilder::new().build();
+/// let rt = QuickJsRuntimeBuilder::new().build();
 /// rt.exe_rt_task_in_event_loop(|q_js_rt| {
 ///     let q_ctx = q_js_rt.get_main_context();
 ///     let obj_ref = q_ctx.eval(Script::new("get_length_test.es", "([1, 2, 3]);")).ok().expect("script failed");
@@ -45,7 +45,7 @@ pub unsafe fn is_array(context: *mut q::JSContext, obj_ref: &JSValueRef) -> bool
 ///     assert_eq!(len, 3);
 /// });
 /// ```
-pub fn get_length_q(q_ctx: &QuickJsContext, arr_ref: &JSValueRef) -> Result<u32, JsError> {
+pub fn get_length_q(q_ctx: &QuickJsRealmAdapter, arr_ref: &JSValueRef) -> Result<u32, JsError> {
     unsafe { get_length(q_ctx.context, arr_ref) }
 }
 
@@ -63,12 +63,12 @@ pub unsafe fn get_length(context: *mut q::JSContext, arr_ref: &JSValueRef) -> Re
 /// Create a new Array
 /// # Example
 /// ```rust
-/// use quickjs_runtime::esruntimebuilder::EsRuntimeBuilder;
+/// use quickjs_runtime::builder::QuickJsRuntimeBuilder;
 /// use hirofa_utils::js_utils::Script;
 /// use quickjs_runtime::quickjs_utils::{arrays, primitives, functions};
 /// use quickjs_runtime::quickjs_utils;
 ///
-/// let rt = EsRuntimeBuilder::new().build();
+/// let rt = QuickJsRuntimeBuilder::new().build();
 /// rt.exe_rt_task_in_event_loop(|q_js_rt| {
 ///     let q_ctx = q_js_rt.get_main_context();
 ///     // create a method to pass our new array to
@@ -78,8 +78,8 @@ pub unsafe fn get_length(context: *mut q::JSContext, arr_ref: &JSValueRef) -> Re
 ///     // add some values
 ///     let val0 = primitives::from_i32(12);
 ///     let val1 = primitives::from_i32(17);
-///     arrays::set_element_q(q_ctx, &arr_ref, 0, val0);
-///     arrays::set_element_q(q_ctx, &arr_ref, 1, val1);
+///     arrays::set_element_q(q_ctx, &arr_ref, 0, &val0);
+///     arrays::set_element_q(q_ctx, &arr_ref, 1, &val1);
 ///     // call the function
 ///     let result_ref = functions::invoke_member_function_q(q_ctx, &quickjs_utils::get_global_q(q_ctx), "create_array_func", vec![arr_ref]).ok().expect("could not invoke function");
 ///     let len = primitives::to_i32(&result_ref).ok().unwrap();
@@ -87,7 +87,7 @@ pub unsafe fn get_length(context: *mut q::JSContext, arr_ref: &JSValueRef) -> Re
 /// });
 /// ```
 
-pub fn create_array_q(q_ctx: &QuickJsContext) -> Result<JSValueRef, JsError> {
+pub fn create_array_q(q_ctx: &QuickJsRealmAdapter) -> Result<JSValueRef, JsError> {
     unsafe { create_array(q_ctx.context) }
 }
 
@@ -105,29 +105,29 @@ pub unsafe fn create_array(context: *mut q::JSContext) -> Result<JSValueRef, JsE
 /// Set a single element in an array
 /// # Example
 /// ```rust
-/// use quickjs_runtime::esruntimebuilder::EsRuntimeBuilder;
+/// use quickjs_runtime::builder::QuickJsRuntimeBuilder;
 /// use hirofa_utils::js_utils::Script;
 /// use quickjs_runtime::quickjs_utils::{arrays, primitives};
 /// use quickjs_runtime::quickjs_utils;
 ///
-/// let rt = EsRuntimeBuilder::new().build();
+/// let rt = QuickJsRuntimeBuilder::new().build();
 /// rt.exe_rt_task_in_event_loop(|q_js_rt| {
 ///     let q_ctx = q_js_rt.get_main_context();
 ///     // get an Array from script
 ///     let arr_ref = q_ctx.eval(Script::new("set_element_test.es", "([1, 2, 3]);")).ok().expect("script failed");
 ///     // add some values
-///     arrays::set_element_q(q_ctx, &arr_ref, 3, primitives::from_i32(12));
-///     arrays::set_element_q(q_ctx, &arr_ref, 4, primitives::from_i32(17));
+///     arrays::set_element_q(q_ctx, &arr_ref, 3, &primitives::from_i32(12));
+///     arrays::set_element_q(q_ctx, &arr_ref, 4, &primitives::from_i32(17));
 ///     // get the length
 ///     let len = arrays::get_length_q(q_ctx, &arr_ref).ok().unwrap();
 ///     assert_eq!(len, 5);
 /// });
 /// ```
 pub fn set_element_q(
-    q_ctx: &QuickJsContext,
+    q_ctx: &QuickJsRealmAdapter,
     array_ref: &JSValueRef,
     index: u32,
-    entry_value_ref: JSValueRef,
+    entry_value_ref: &JSValueRef,
 ) -> Result<(), JsError> {
     unsafe { set_element(q_ctx.context, array_ref, index, entry_value_ref) }
 }
@@ -138,7 +138,7 @@ pub unsafe fn set_element(
     context: *mut q::JSContext,
     array_ref: &JSValueRef,
     index: u32,
-    entry_value_ref: JSValueRef,
+    entry_value_ref: &JSValueRef,
 ) -> Result<(), JsError> {
     let entry_value_ref = entry_value_ref;
 
@@ -158,12 +158,12 @@ pub unsafe fn set_element(
 /// Get a single element from an array
 /// # Example
 /// ```rust
-/// use quickjs_runtime::esruntimebuilder::EsRuntimeBuilder;
+/// use quickjs_runtime::builder::QuickJsRuntimeBuilder;
 /// use hirofa_utils::js_utils::Script;
 /// use quickjs_runtime::quickjs_utils::{arrays, primitives};
 /// use quickjs_runtime::quickjs_utils;
 ///
-/// let rt = EsRuntimeBuilder::new().build();
+/// let rt = QuickJsRuntimeBuilder::new().build();
 /// rt.exe_rt_task_in_event_loop(|q_js_rt| {
 ///     let q_ctx = q_js_rt.get_main_context();
 ///     // get an Array from script
@@ -176,7 +176,7 @@ pub unsafe fn set_element(
 /// });
 /// ```
 pub fn get_element_q(
-    q_ctx: &QuickJsContext,
+    q_ctx: &QuickJsRealmAdapter,
     array_ref: &JSValueRef,
     index: u32,
 ) -> Result<JSValueRef, JsError> {
@@ -206,15 +206,13 @@ pub unsafe fn get_element(
 
 #[cfg(test)]
 pub mod tests {
-    use crate::esruntime::tests::init_test_rt;
-    use crate::esruntime::EsRuntime;
+    use crate::facades::tests::init_test_rt;
     use crate::quickjs_utils::arrays::{create_array_q, get_element_q, set_element_q};
     use crate::quickjs_utils::objects;
-    use std::sync::Arc;
 
     #[test]
     fn test_array() {
-        let rt: Arc<EsRuntime> = init_test_rt();
+        let rt = init_test_rt();
         rt.exe_rt_task_in_event_loop(|q_js_rt| {
             let q_ctx = q_js_rt.get_main_context();
             let arr = create_array_q(q_ctx).ok().unwrap();
@@ -223,7 +221,7 @@ pub mod tests {
             let a = objects::create_object_q(q_ctx).ok().unwrap();
             assert_eq!(1, a.get_ref_count());
 
-            set_element_q(q_ctx, &arr, 0, a.clone()).ok().unwrap();
+            set_element_q(q_ctx, &arr, 0, &a).ok().unwrap();
             assert_eq!(2, a.get_ref_count());
 
             let a2 = get_element_q(q_ctx, &arr, 0).ok().unwrap();
