@@ -2,8 +2,8 @@
 
 use crate::quickjs_utils::properties::JSPropertyEnumRef;
 use crate::quickjs_utils::{atoms, functions, get_constructor, get_global};
-use crate::quickjscontext::QuickJsContext;
-use crate::quickjsruntime::{make_cstring, QuickJsRuntime};
+use crate::quickjsrealmadapter::QuickJsRealmAdapter;
+use crate::quickjsruntimeadapter::{make_cstring, QuickJsRuntimeAdapter};
 use crate::valueref::JSValueRef;
 use hirofa_utils::js_utils::JsError;
 use libquickjs_sys as q;
@@ -12,9 +12,9 @@ use libquickjs_sys as q;
 /// this is used to get nested object properties which are used as namespaces
 /// # Example
 /// ```rust
-/// use quickjs_runtime::esruntimebuilder::EsRuntimeBuilder;
+/// use quickjs_runtime::builder::QuickJsRuntimeBuilder;
 /// use quickjs_runtime::quickjs_utils::objects::get_namespace_q;
-/// let rt = EsRuntimeBuilder::new().build();
+/// let rt = QuickJsRuntimeBuilder::new().build();
 /// rt.exe_rt_task_in_event_loop(|q_js_rt| {
 ///     let q_ctx = q_js_rt.get_main_context();
 ///     let ns_obj = get_namespace_q(q_ctx, vec!["com", "hirofa", "examplepackage"], true).ok().unwrap();
@@ -22,7 +22,7 @@ use libquickjs_sys as q;
 /// })
 /// ```
 pub fn get_namespace_q(
-    context: &QuickJsContext,
+    context: &QuickJsRealmAdapter,
     namespace: Vec<&str>,
     create_if_absent: bool,
 ) -> Result<JSValueRef, JsError> {
@@ -72,7 +72,7 @@ pub unsafe fn get_namespace(
 pub unsafe fn construct_object(
     ctx: *mut q::JSContext,
     constructor_ref: &JSValueRef,
-    args: Vec<JSValueRef>,
+    args: &[&JSValueRef],
 ) -> Result<JSValueRef, JsError> {
     let arg_count = args.len() as i32;
 
@@ -85,10 +85,10 @@ pub unsafe fn construct_object(
         qargs.as_mut_ptr(),
     );
 
-    let res_ref = JSValueRef::new(ctx, res, false, true, "call_function result");
+    let res_ref = JSValueRef::new(ctx, res, false, true, "construct_object result");
 
     if res_ref.is_exception() {
-        if let Some(ex) = QuickJsContext::get_exception(ctx) {
+        if let Some(ex) = QuickJsRealmAdapter::get_exception(ctx) {
             Err(ex)
         } else {
             Err(JsError::new_str(
@@ -101,7 +101,7 @@ pub unsafe fn construct_object(
 }
 
 /// create a new simple object, e.g. `let obj = {};`
-pub fn create_object_q(q_ctx: &QuickJsContext) -> Result<JSValueRef, JsError> {
+pub fn create_object_q(q_ctx: &QuickJsRealmAdapter) -> Result<JSValueRef, JsError> {
     unsafe { create_object(q_ctx.context) }
 }
 
@@ -119,7 +119,7 @@ pub unsafe fn create_object(context: *mut q::JSContext) -> Result<JSValueRef, Js
 
 /// set a property in an object, like `obj[propName] = val;`
 pub fn set_property_q(
-    q_ctx: &QuickJsContext,
+    q_ctx: &QuickJsRealmAdapter,
     obj_ref: &JSValueRef,
     prop_name: &str,
     prop_ref: &JSValueRef,
@@ -160,11 +160,11 @@ pub unsafe fn set_property(
 /// * q::JS_PROP_AUTOINIT
 /// # Example
 /// ```rust
-/// use quickjs_runtime::esruntimebuilder::EsRuntimeBuilder;
+/// use quickjs_runtime::builder::QuickJsRuntimeBuilder;
 /// use quickjs_runtime::quickjs_utils::objects::{create_object_q, set_property2_q};
 /// use quickjs_runtime::quickjs_utils::primitives::from_i32;
 /// use libquickjs_sys as q;
-/// let rt = EsRuntimeBuilder::new().build();
+/// let rt = QuickJsRuntimeBuilder::new().build();
 /// rt.exe_rt_task_in_event_loop(|q_js_rt| {
 ///    let q_ctx = q_js_rt.get_main_context();
 ///    let obj = create_object_q(q_ctx).ok().unwrap();
@@ -174,7 +174,7 @@ pub unsafe fn set_property(
 /// })
 /// ```                         
 pub fn set_property2_q(
-    q_ctx: &QuickJsContext,
+    q_ctx: &QuickJsRealmAdapter,
     obj_ref: &JSValueRef,
     prop_name: &str,
     prop_ref: &JSValueRef,
@@ -256,7 +256,7 @@ pub unsafe fn set_property2(
 /// rt.eval_sync(Script::new("define_getter_setter_q.es", "testObj431.someProperty = 'hello prop';")).ok().expect("script failed");
 /// ```
 pub fn define_getter_setter_q(
-    q_ctx: &QuickJsContext,
+    q_ctx: &QuickJsRealmAdapter,
     obj_ref: &JSValueRef,
     prop_name: &str,
     getter_func_ref: &JSValueRef,
@@ -297,9 +297,9 @@ pub unsafe fn define_getter_setter(
 
     log::trace!("objects::define_getter_setter 1");
 
-    debug_assert!(functions::is_function(context, &getter_func_ref));
+    debug_assert!(functions::is_function(context, getter_func_ref));
     log::trace!("objects::define_getter_setter 2");
-    debug_assert!(functions::is_function(context, &setter_func_ref));
+    debug_assert!(functions::is_function(context, setter_func_ref));
     log::trace!("objects::define_getter_setter 3");
 
     let prop_atom = atoms::from_string(context, prop_name)?;
@@ -318,7 +318,7 @@ pub unsafe fn define_getter_setter(
     log::trace!("objects::define_getter_setter 5 {}", res);
 
     if res != 0 {
-        if let Some(err) = QuickJsContext::get_exception(context) {
+        if let Some(err) = QuickJsRealmAdapter::get_exception(context) {
             Err(err)
         } else {
             Err(JsError::new_str(
@@ -332,7 +332,7 @@ pub unsafe fn define_getter_setter(
 
 /// get a property from an object by name
 pub fn get_property_q(
-    q_ctx: &QuickJsContext,
+    q_ctx: &QuickJsRealmAdapter,
     obj_ref: &JSValueRef,
     prop_name: &str,
 ) -> Result<JSValueRef, JsError> {
@@ -371,7 +371,7 @@ pub unsafe fn get_property(
 
 /// get the property names of an object
 pub fn get_own_property_names_q(
-    q_ctx: &QuickJsContext,
+    q_ctx: &QuickJsRealmAdapter,
     obj_ref: &JSValueRef,
 ) -> Result<JSPropertyEnumRef, JsError> {
     unsafe { get_own_property_names(q_ctx.context, obj_ref) }
@@ -405,7 +405,7 @@ pub unsafe fn get_own_property_names(
 
 /// get the names of all properties of an object
 pub fn get_property_names_q(
-    q_ctx: &QuickJsContext,
+    q_ctx: &QuickJsRealmAdapter,
     obj_ref: &JSValueRef,
 ) -> Result<Vec<String>, JsError> {
     unsafe { get_property_names(q_ctx.context, obj_ref) }
@@ -431,7 +431,7 @@ pub unsafe fn get_property_names(
 }
 
 pub fn traverse_properties_q<V, R>(
-    q_ctx: &QuickJsContext,
+    q_ctx: &QuickJsRealmAdapter,
     obj_ref: &JSValueRef,
     visitor: V,
 ) -> Result<Vec<R>, JsError>
@@ -441,6 +441,17 @@ where
     unsafe { traverse_properties(q_ctx.context, obj_ref, visitor) }
 }
 
+pub fn traverse_properties_q_mut<V, R>(
+    q_ctx: &QuickJsRealmAdapter,
+    obj_ref: &JSValueRef,
+    visitor: V,
+) -> Result<(), JsError>
+where
+    V: FnMut(&str, &JSValueRef) -> Result<R, JsError>,
+{
+    unsafe { traverse_properties_mut(q_ctx.context, obj_ref, visitor) }
+}
+
 /// # Safety
 /// When passing a context pointer please make sure the corresponding QuickJsContext is still valid
 pub unsafe fn traverse_properties<V, R>(
@@ -485,8 +496,78 @@ where
     Ok(result)
 }
 
+/// # Safety
+/// When passing a context pointer please make sure the corresponding QuickJsContext is still valid
+pub unsafe fn traverse_properties_mut<V, R>(
+    context: *mut q::JSContext,
+    obj_ref: &JSValueRef,
+    mut visitor: V,
+) -> Result<(), JsError>
+where
+    V: FnMut(&str, &JSValueRef) -> Result<R, JsError>,
+{
+    let enum_ref = get_own_property_names(context, obj_ref)?;
+
+    for index in 0..enum_ref.len() {
+        let atom = enum_ref.get_atom_raw(index) as q::JSAtom;
+        let prop_name = atoms::to_str(context, &atom)?;
+
+        let raw_value = q::JS_GetPropertyInternal(
+            context,
+            *obj_ref.borrow_value(),
+            atom,
+            *obj_ref.borrow_value(),
+            0,
+        );
+        let prop_val_ref = JSValueRef::new(
+            context,
+            raw_value,
+            false,
+            true,
+            "objects::traverse_properties raw_value",
+        );
+        if prop_val_ref.is_exception() {
+            return Err(JsError::new_str("Could not get object property"));
+        }
+
+        visitor(prop_name, &prop_val_ref)?;
+    }
+
+    Ok(())
+}
+
+pub fn get_prototype_of_q(
+    q_ctx: &QuickJsRealmAdapter,
+    obj_ref: &JSValueRef,
+) -> Result<JSValueRef, JsError> {
+    unsafe { get_prototype_of(q_ctx.context, obj_ref) }
+}
+
+/// Object.prototypeOf
+/// # Safety
+/// please ensure the JSContext is valid and remains valid while using this function
+pub unsafe fn get_prototype_of(
+    ctx: *mut q::JSContext,
+    obj_ref: &JSValueRef,
+) -> Result<JSValueRef, JsError> {
+    let raw = q::JS_GetPrototype(ctx, *obj_ref.borrow_value());
+    let pt_ref = JSValueRef::new(ctx, raw, false, true, "object::get_prototype_of_q");
+
+    if pt_ref.is_exception() {
+        if let Some(ex) = QuickJsRealmAdapter::get_exception(ctx) {
+            Err(ex)
+        } else {
+            Err(JsError::new_str(
+                "get_prototype_of_q failed but could not get ex",
+            ))
+        }
+    } else {
+        Ok(pt_ref)
+    }
+}
+
 pub fn is_instance_of_q(
-    q_ctx: &QuickJsContext,
+    q_ctx: &QuickJsRealmAdapter,
     obj_ref: &JSValueRef,
     constructor_ref: &JSValueRef,
 ) -> bool {
@@ -512,7 +593,7 @@ pub unsafe fn is_instance_of(
 }
 
 pub fn is_instance_of_by_name_q(
-    context: &QuickJsContext,
+    context: &QuickJsRealmAdapter,
     obj_ref: &JSValueRef,
     constructor_name: &str,
 ) -> Result<bool, JsError> {
@@ -539,7 +620,7 @@ pub unsafe fn is_instance_of_by_name(
         Ok(true)
     } else {
         // todo check if context is not __main__
-        QuickJsRuntime::do_with(|q_js_rt| {
+        QuickJsRuntimeAdapter::do_with(|q_js_rt| {
             let main_ctx = q_js_rt.get_main_context();
             let main_constructor_ref = get_constructor(main_ctx.context, constructor_name)?;
             if is_instance_of(main_ctx.context, obj_ref, &main_constructor_ref) {
@@ -553,7 +634,7 @@ pub unsafe fn is_instance_of_by_name(
 
 #[cfg(test)]
 pub mod tests {
-    use crate::esruntime::tests::init_test_rt;
+    use crate::facades::tests::init_test_rt;
     use crate::quickjs_utils::objects::{
         create_object_q, get_property_names_q, get_property_q, set_property_q,
     };
